@@ -1,9 +1,31 @@
 from django.contrib.contenttypes.models import ContentType
 from django_dynamic_fixture import G, N
-from entity.models import Entity, EntityKind
+from django.test import TestCase
+from entity.sync import sync_entities
 
-from .models import Account, Team, TeamGroup, Competitor
-from .utils import EntityTestCase
+from entity.signal_handlers import turn_off_syncing, turn_on_syncing
+
+from entity.models import (
+    Entity, EntityKind, EntityRelationship, EntityGroup, EntityGroupMembership, get_entities_by_kind
+)
+
+from entity.tests.models import Account, Team, TeamGroup, Competitor
+from entity.tests.utils import EntityTestCase
+
+
+class EntityKindManagerTest(EntityTestCase):
+    """
+    Tests the active and all entity kind managers.
+    """
+    def test_only_active(self):
+        G(EntityKind, is_active=False)
+        active_ek = G(EntityKind, is_active=True)
+        self.assertEquals([active_ek], list(EntityKind.objects.all()))
+
+    def test_all_objects(self):
+        inactive_ek = G(EntityKind, is_active=False)
+        active_ek = G(EntityKind, is_active=True)
+        self.assertEquals(set([active_ek, inactive_ek]), set(EntityKind.all_objects.all()))
 
 
 class EntityKindTest(EntityTestCase):
@@ -12,7 +34,15 @@ class EntityKindTest(EntityTestCase):
     """
     def test_unicode(self):
         ek = N(EntityKind, display_name='hello')
-        self.assertEquals(unicode(ek), u'hello')
+        self.assertEquals(u'{0}'.format(ek), u'hello')
+
+    def test_regular_delete(self):
+        """
+        Regular deletion should deactivate the entity kind.
+        """
+        ek = G(EntityKind, is_active=True)
+        ek.delete()
+        self.assertFalse(ek.is_active)
 
 
 class TestActiveEntityManager(EntityTestCase):
@@ -704,5 +734,341 @@ class TestEntityModel(EntityTestCase):
         """
         account = Account.objects.create(email='hi')
         entity = Entity.objects.get_for_obj(account)
-        entity_unicode = entity.__unicode__()
+        entity_unicode = entity.__str__()
         self.assertEquals(entity_unicode, 'hi')
+
+
+class EntityGroupAllEntitiesTest(EntityTestCase):
+    def setUp(self):
+        super(EntityGroupAllEntitiesTest, self).setUp()
+        # Create three super entities, each with two sub
+        # entities. THere are two entity kinds. Super entity 1 has two
+        # sub entities of the first kind, super entity 2 has subs of
+        # two different kinds, and super entity three has two of the
+        # second kind.
+        self.kind1, self.kind2 = G(EntityKind), G(EntityKind)
+        self.super_entities = [G(Entity) for _ in range(3)]
+        self.sub_entities = [
+            G(Entity, entity_kind=k)
+            for k in [self.kind1] * 3 + [self.kind2] * 3
+        ]
+        for i, sub in enumerate(self.sub_entities):
+            sup = self.super_entities[i // 2]
+            G(EntityRelationship, sub_entity=sub, super_entity=sup)
+
+        self.group = G(EntityGroup)
+
+    def test_individual_entities_returned(self):
+        e = self.super_entities[0]
+        G(EntityGroupMembership, entity_group=self.group, entity=e, sub_entity_kind=None)
+        result = list(self.group.all_entities().order_by('id'))
+        expected = [e]
+        self.assertEqual(result, expected)
+
+    def test_sub_entity_group_entities_returned(self):
+        e = self.super_entities[0]
+        G(
+            EntityGroupMembership, entity_group=self.group,
+            entity=e, sub_entity_kind=self.kind1)
+        result = list(self.group.all_entities().order_by('id'))
+        expected = self.sub_entities[0:2]
+        self.assertEqual(result, expected)
+
+    def test_sub_entity_group_entities_filters_by_kind(self):
+        e = self.super_entities[1]
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e, sub_entity_kind=self.kind1)
+        result = list(self.group.all_entities().order_by('id'))
+        expected = [self.sub_entities[2]]
+        self.assertEqual(result, expected)
+
+    def test_combined_returned(self):
+        e = self.super_entities[1]
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e, sub_entity_kind=self.kind1)
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e, sub_entity_kind=None)
+        result = list(self.group.all_entities().order_by('id'))
+        expected = [e, self.sub_entities[2]]
+
+        self.assertEqual(result, expected)
+
+    def test_all_entities_of_a_kind_returned(self):
+        """
+        When a group has a sub_entity_kind but a null entity, it should return all entities of that kind
+        """
+        G(EntityGroupMembership, entity_group=self.group, sub_entity_kind=self.kind1)
+        result = list(self.group.all_entities().order_by('id'))
+        expected = [self.sub_entities[0], self.sub_entities[1], self.sub_entities[2]]
+
+        self.assertEqual(result, expected)
+
+    def test_filters_groups(self):
+        other_group = G(EntityGroup)
+        e = self.super_entities[1]
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e, sub_entity_kind=self.kind1)
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e, sub_entity_kind=None)
+        G(EntityGroupMembership, entity_group=other_group,
+          entity=e, sub_entity_kind=None)
+        result = self.group.all_entities().count()
+        expected = 2
+        self.assertEqual(result, expected)
+
+    def test_number_of_queries(self):
+        e1 = self.super_entities[1]
+        e2 = self.super_entities[2]
+        sub1 = self.sub_entities[0]
+
+        # Individual memberships
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e1, sub_entity_kind=None)
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=sub1, sub_entity_kind=None)
+
+        # Group memberships
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e1, sub_entity_kind=self.kind1)
+        G(EntityGroupMembership, entity_group=self.group,
+          entity=e2, sub_entity_kind=self.kind2)
+
+        with self.assertNumQueries(3):
+            list(self.group.all_entities())
+
+
+class EntityGroupAddEntityTest(EntityTestCase):
+    def test_adds_entity(self):
+        group = G(EntityGroup)
+        e = G(Entity)
+        membership = group.add_entity(e)
+        count = EntityGroupMembership.objects.filter(entity_group=group).count()
+        expected = 1
+        self.assertEqual(count, expected)
+        self.assertIsInstance(membership, EntityGroupMembership)
+
+
+class EntityGroupBulkAddEntitiesTest(EntityTestCase):
+    def test_bulk_adds(self):
+        group = G(EntityGroup)
+        e1, e2 = G(Entity), G(Entity)
+        k = G(EntityKind)
+        to_add = [(e1, k), (e2, None)]
+        membership = group.bulk_add_entities(to_add)
+        count = EntityGroupMembership.objects.filter(entity_group=group).count()
+        expected = 2
+        self.assertEqual(count, expected)
+        self.assertIsInstance(membership[0], EntityGroupMembership)
+
+    def test_bulk_add_none_entity(self):
+        group = EntityGroup.objects.create()
+        e = G(Entity)
+        k = G(EntityKind)
+        group.bulk_add_entities([(None, k), (e, k), (e, None)])
+        count = EntityGroupMembership.objects.filter(entity_group=group).count()
+        self.assertEqual(count, 3)
+
+
+class EntityGroupRemoveEntityTest(EntityTestCase):
+    def setUp(self):
+        self.group = G(EntityGroup)
+        self.e1, self.e2 = G(Entity), G(Entity)
+        self.k = G(EntityKind)
+        to_add = [(self.e1, self.k), (self.e2, None)]
+        self.group.bulk_add_entities(to_add)
+
+    def test_removes_only_selected_no_entity_kind(self):
+        self.group.remove_entity(self.e2)
+        member = EntityGroupMembership.objects.get(entity_group=self.group)
+        expected = self.k
+        self.assertEqual(member.sub_entity_kind, expected)
+
+    def test_removes_only_selected_with_entity_kind(self):
+        self.group.remove_entity(self.e1, self.k)
+        member = EntityGroupMembership.objects.get(entity_group=self.group)
+        expected = None
+        self.assertEqual(member.sub_entity_kind, expected)
+
+
+class EntityGroupBulkRemoveEntitiesTest(EntityTestCase):
+    def setUp(self):
+        self.group = G(EntityGroup)
+        self.e1, self.e2, self.e3 = G(Entity), G(Entity), G(Entity)
+        self.k = G(EntityKind)
+        to_add = [(self.e1, self.k), (self.e2, None), (self.e3, self.k), (self.e3, None)]
+        self.group.bulk_add_entities(to_add)
+
+    def test_only_removes_selected(self):
+        self.group.bulk_remove_entities([(self.e3, self.k), (self.e2, None)])
+        count = EntityGroupMembership.objects.filter(entity_group=self.group).count()
+        expected = 2
+        self.assertEqual(count, expected)
+
+    def test_num_queries(self):
+        with self.assertNumQueries(2):
+            self.group.bulk_remove_entities([(self.e3, self.k), (self.e2, None)])
+
+
+class EntityGroupBulkOverwriteEntitiesTest(EntityTestCase):
+    def setUp(self):
+        self.group = G(EntityGroup)
+        self.e1, self.e2, self.e3 = G(Entity), G(Entity), G(Entity)
+        self.k = G(EntityKind)
+        to_add = [(self.e1, self.k), (self.e2, None), (self.e3, self.k), (self.e3, None)]
+        self.group.bulk_add_entities(to_add)
+
+    def test_overwrites(self):
+        to_overwrite = [(self.e1, None), (self.e2, self.k)]
+        self.group.bulk_overwrite(to_overwrite)
+        count = EntityGroupMembership.objects.filter(entity_group=self.group).count()
+        new_members = EntityGroupMembership.objects.values_list('entity', 'sub_entity_kind')
+        expected = 2
+        self.assertEqual(count, expected)
+        self.assertIn((self.e1.id, None), new_members)
+
+    def test_bulk_overwrite_none_entity(self):
+        group = EntityGroup.objects.create()
+        group.bulk_overwrite([(None, self.k)])
+        count = EntityGroupMembership.objects.filter(entity_group=group).count()
+        self.assertEqual(count, 1)
+
+
+class EntityGroupTest(TestCase):
+
+    def test_get_all_entities(self):
+        turn_off_syncing()
+
+        account_type = ContentType.objects.get_for_model(Account)
+        team_type = ContentType.objects.get_for_model(Team)
+
+        # Set up teams
+        teams = Team.objects.bulk_create([
+            Team()
+            for i in range(0, 3)
+        ])
+
+        # Set up accounts
+        Account.objects.bulk_create([
+            Account(team=teams[i % 3])
+            for i in range(0, 20)
+        ])
+
+        # Turn on syncing and do a sync
+        turn_on_syncing()
+
+        sync_entities()
+
+        account_entities = list(Entity.all_objects.filter(entity_type=account_type).order_by('entity_id'))
+        account_kind = account_entities[0].entity_kind
+
+        team_entities = list(Entity.all_objects.filter(entity_type=team_type).order_by('entity_id'))
+        team_kind = team_entities[0].entity_kind
+
+        # Create groups
+        EntityGroup.objects.bulk_create([
+            EntityGroup()
+            for i in range(0, 6)
+        ])
+
+        # Refresh for django 1.9 because bulk create does not return ids
+        entity_groups = list(EntityGroup.objects.order_by('id'))
+
+        # Set up individual entity groups
+        entity_groups[0].bulk_add_entities([
+            [account_entities[0], None],
+            [account_entities[1], None],
+            [account_entities[2], None],
+            [account_entities[3], None],
+        ])
+        entity_groups[1].bulk_add_entities([
+            [account_entities[2], None],
+            [account_entities[3], None],
+            [account_entities[4], None],
+            [account_entities[5], None],
+        ])
+
+        # Set up sub entity kind of super entity groups
+        entity_groups[2].bulk_add_entities([
+            [team_entities[0], account_kind],
+            [team_entities[1], account_kind],
+        ])
+
+        entity_groups[3].bulk_add_entities([
+            [team_entities[0], account_kind],
+        ])
+
+        # Set up sub entity kind groups
+        # This group has two copies of the same set of all accounts as well as all teams
+        entity_groups[4].bulk_add_entities([
+            [None, account_kind],
+            [None, account_kind],
+            [None, team_kind],
+        ])
+
+        # This group has the same copy of all accounts
+        entity_groups[5].bulk_add_entities([
+            [None, account_kind],
+        ])
+
+        with self.assertNumQueries(3):
+            membership_cache = EntityGroup.objects.get_membership_cache()
+            entities_by_kind = get_entities_by_kind(membership_cache=membership_cache)
+
+            for entity_group in entity_groups:
+                entity_group.get_all_entities(membership_cache, entities_by_kind)
+
+        with self.assertNumQueries(3):
+            get_entities_by_kind()
+
+        # Make sure to hit the no group cache case
+        self.assertEqual(entity_groups[0].get_all_entities(membership_cache={1000: []}), set())
+
+    def test_get_all_entities_active(self):
+        """
+        Makes sure only active entity ids are returned
+        """
+        turn_off_syncing()
+
+        account_type = ContentType.objects.get_for_model(Account)
+
+        # Set up teams
+        teams = Team.objects.bulk_create([
+            Team()
+            for i in range(0, 3)
+        ])
+
+        # Set up accounts
+        accounts = Account.objects.bulk_create([
+            Account(team=teams[i % 3])
+            for i in range(0, 20)
+        ])
+        accounts[0].is_active = False
+        accounts[0].save()
+
+        # Turn on syncing and do a sync
+        turn_on_syncing()
+
+        sync_entities()
+
+        account_entities = list(Entity.all_objects.filter(entity_type=account_type).order_by('entity_id'))
+
+        # Create groups
+        EntityGroup.objects.bulk_create([
+            EntityGroup()
+            for i in range(0, 6)
+        ])
+
+        # Refresh for django 1.9 because bulk create does not return ids
+        entity_groups = list(EntityGroup.objects.order_by('id'))
+
+        # Set up individual entity groups
+        entity_groups[0].bulk_add_entities([
+            [account_entities[0], None],
+            [account_entities[1], None],
+            [account_entities[2], None],
+            [account_entities[3], None],
+        ])
+
+        self.assertEqual(len(entity_groups[0].get_all_entities()), 3)
+        self.assertEqual(len(entity_groups[0].get_all_entities(is_active=False)), 1)
+        self.assertEqual(len(entity_groups[0].get_all_entities(is_active=None)), 4)
